//
// Created by philipp on 08.05.18.
//

#include "../include/Logger.h"
#include <iostream>

namespace pc
{

Logger::Logger(Logger::PrivateTag, networking::Networking & net, const std::string & host)
    : net(net), host(host)
{
    client = LoggingClient::create(net, MAX_MESSAGE_SIZE);
    timer = networking::time::Timer::create(net);
}

void Logger::start()
{
    auto self = shared_from_this();
    auto state = std::make_shared<AsyncState>(self);
    log(state);
}

void Logger::stop()
{
    timer->stop();
    client->stop();
}

void Logger::log(AsyncState::Ptr state)
{
    using namespace std::chrono_literals;
    std::string emptyRequest;
    client->asyncCall(
        emptyRequest, host, PORT, 10s,
        [state](const auto & error, auto & responseMessage)
        {
            if (!responseMessage.empty())
                std::cout << responseMessage;

            state->self->timer->startTimeout(1ms, [state]
            { state->self->log(state); });
        });
}

}

